@@ -4,7 +4,7 @@ mkdir -p m4
 autoreconf -vfi
 ./configure --disable-dependency-tracking
 if [ "$TRAVIS_OS_NAME" == "osx" ] || [ "$CC" == "clang" ]; then
-  make CFLAGS="-fsanitize=address -g"
+  make CFLAGS="-fsanitize=address -g" LDFLAGS="-fsanitize=address"
 else
   make
 fi
